"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const crud_1 = require("@nestjsx/crud");
const util_1 = require("@nestjsx/util");
class TypeOrmMongodbCrudService extends crud_1.CrudService {
    constructor(repo) {
        super();
        this.repo = repo;
        this.entityRelationsHash = {};
        this.entityColumnsHash = {};
        this.onInitMapEntityColumns();
        this.onInitMapRelations();
    }
    createMany(req, dto) {
        return undefined;
    }
    createOne(req, dto) {
        return undefined;
    }
    deleteOne(req) {
        return undefined;
    }
    async getMany(req) {
        const { parsed, options } = req;
        const cursor = this.createCursor(parsed, options);
        return this.doGetMany(cursor, parsed, options);
    }
    getOne(req) {
        return this.getOneOrFail(req);
    }
    replaceOne(req, dto) {
        return undefined;
    }
    updateOne(req, dto) {
        return undefined;
    }
    onInitMapRelations() {
        this.entityRelationsHash = this.repo.metadata.relations.reduce((hash, curr) => ({
            ...hash,
            [curr.propertyName]: {
                name: curr.propertyName,
                columns: curr.inverseEntityMetadata.columns.map((col) => col.propertyName),
                primaryColumns: curr.inverseEntityMetadata.primaryColumns.map((col) => col.propertyName),
            },
        }), {});
    }
    onInitMapEntityColumns() {
        this.entityColumns = this.repo.metadata.columns.map((prop) => {
            if (prop.embeddedMetadata) {
                this.entityColumnsHash[prop.propertyPath] = true;
                return prop.propertyPath;
            }
            this.entityColumnsHash[prop.propertyName] = true;
            return prop.propertyName;
        });
        this.entityPrimaryColumns = this.repo.metadata.columns
            .filter((prop) => prop.isPrimary)
            .map((prop) => prop.propertyName);
    }
    async getOneOrFail(req) {
        const { parsed, options } = req;
        const cursor = await this.createCursor(parsed, options);
        const found = await cursor.toArray();
        if (found.length !== 1) {
            this.throwNotFoundException(this.repo.metadata.targetName);
        }
        return found[0];
    }
    createCursor(parsed, options, many = true) {
        const cursor = this.repo.createCursor();
        const select = this.getSelect(parsed, options.query);
        cursor.project(select);
        const filter = {};
        this.setSearchCondition(filter, parsed.search);
        cursor.filter(filter);
        if (many) {
            const sort = this.getSort(parsed, options.query);
            if (sort) {
                cursor.sort(sort);
            }
            const limit = this.getTake(parsed, options.query);
            if (limit && isFinite(limit)) {
                cursor.limit(limit);
            }
            const skip = this.getSkip(parsed, limit);
            if (skip && isFinite(skip)) {
                cursor.skip(skip);
            }
        }
        return cursor;
    }
    async doGetMany(cursor, query, options) {
        if (this.decidePagination(query, options)) {
            const total = await cursor.count(false);
            const data = await cursor.toArray();
            const limit = this.getTake(query, options.query);
            const offset = this.getSkip(query, limit);
            return this.createPageInfo(data, total, limit || total, offset || 0);
        }
        return cursor.toArray();
    }
    createPageInfo(data, total, limit, offset) {
        const test = {
            data,
            count: data.length,
            total,
            page: Math.floor(offset / limit) + 1,
            pageCount: limit && total
                ? Math.ceil(total / limit)
                :
                    undefined,
        };
        return test;
    }
    getSelect(query, options) {
        const allowed = this.getAllowedColumns(this.entityColumns, options);
        const columns = query.fields && query.fields.length
            ? query.fields.filter((field) => allowed.some((col) => field === col))
            : allowed;
        const select = Array.from(new Set([
            ...(options.persist && options.persist.length ? options.persist : []),
            ...columns,
            ...this.entityPrimaryColumns,
        ]));
        const projection = {};
        select.forEach(s => {
            projection[s] = 1;
        });
        return projection;
    }
    getSort(query, options) {
        return query.sort && query.sort.length
            ? this.mapSort(query.sort)
            : options.sort && options.sort.length
                ? this.mapSort(options.sort)
                : null;
    }
    mapSort(sort) {
        const params = {};
        for (let i = 0; i < sort.length; i++) {
            params[sort[i].field] = sort[i].order === 'ASC' ? 1 : -1;
        }
        return params;
    }
    getAllowedColumns(columns, options) {
        return (!options.exclude || !options.exclude.length) &&
            (!options.allow || !options.allow.length)
            ? columns
            : columns.filter((column) => (options.exclude && options.exclude.length
                ? !options.exclude.some((col) => col === column)
                : true) &&
                (options.allow && options.allow.length
                    ? options.allow.some((col) => col === column)
                    : true));
    }
    setSearchCondition(filter, search, condition = '$and') {
        if (util_1.isObject(search)) {
            const keys = util_1.objKeys(search);
            if (keys.length) {
                if (util_1.isArrayFull(search.$and)) {
                    if (search.$and.length === 1) {
                        this.setSearchCondition(filter, search.$and[0], condition);
                    }
                    else {
                        this.builderAddBrackets(filter, condition, (and) => search.$and.forEach((item) => {
                            this.setSearchCondition(and, item, '$and');
                        }));
                    }
                }
                else if (util_1.isArrayFull(search.$or)) {
                    if (keys.length === 1) {
                        if (search.$or.length === 1) {
                            this.setSearchCondition(filter, search.$or[0], condition);
                        }
                        else {
                            this.builderAddBrackets(filter, condition, (or) => {
                                search.$or.forEach((item) => {
                                    this.setSearchCondition(or, item, '$or');
                                });
                            });
                        }
                    }
                    else {
                        this.builderAddBrackets(filter, condition, (qb) => {
                            keys.forEach((field) => {
                                if (field !== '$or') {
                                    const value = search[field];
                                    if (!util_1.isObject(value)) {
                                        this.builderSetWhere(qb, '$and', field, value);
                                    }
                                    else {
                                        this.setSearchFieldObjectCondition(qb, '$and', field, value);
                                    }
                                }
                                else {
                                    if (search.$or.length === 1) {
                                        this.setSearchCondition(filter, search.$or[0], '$and');
                                    }
                                    else {
                                        this.builderAddBrackets(qb, '$and', (and) => {
                                            search.$or.forEach((item) => {
                                                this.setSearchCondition(and, item, '$or');
                                            });
                                        });
                                    }
                                }
                            });
                        });
                    }
                }
                else {
                    if (keys.length === 1) {
                        const field = keys[0];
                        const value = search[field];
                        if (!util_1.isObject(value)) {
                            this.builderSetWhere(filter, condition, field, value);
                        }
                        else {
                            this.setSearchFieldObjectCondition(filter, condition, field, value);
                        }
                    }
                }
            }
        }
    }
    setSearchFieldObjectCondition(filter, condition, field, object) {
        if (util_1.isObject(object)) {
            const operators = util_1.objKeys(object);
            if (operators.length === 1) {
                const operator = operators[0];
                const value = object[operator];
                if (util_1.isObject(object.$or)) {
                    const orKeys = util_1.objKeys(object.$or);
                    this.setSearchFieldObjectCondition(filter, orKeys.length === 1 ? condition : '$or', field, object.$or);
                }
                else {
                    this.builderSetWhere(filter, condition, field, value, operator);
                }
            }
            else {
                if (operators.length > 1) {
                    this.builderAddBrackets(filter, condition, (qb) => {
                        operators.forEach((operator) => {
                            const value = object[operator];
                            if (operator !== '$or') {
                                this.builderSetWhere(qb, condition, field, value, operator);
                            }
                            else {
                                const orKeys = util_1.objKeys(object.$or);
                                if (orKeys.length === 1) {
                                    this.setSearchFieldObjectCondition(qb, condition, field, object.$or);
                                }
                                else {
                                    this.builderAddBrackets(qb, condition, (qb2) => {
                                        this.setSearchFieldObjectCondition(qb2, '$or', field, object.$or);
                                    });
                                }
                            }
                        });
                    });
                }
            }
        }
    }
    builderAddBrackets(filter, condition, children) {
        const brackets = [];
        children(brackets);
        if (!brackets.length) {
            return;
        }
        if (Array.isArray(filter)) {
            filter.push({ [condition]: brackets });
        }
        else {
            filter[condition] = brackets;
        }
    }
    builderSetWhere(filter, condition, field, value, operator = '$eq') {
        const args = [
            { field, operator: util_1.isNull(value) ? '$isnull' : operator, value },
            filter,
        ];
        const fn = condition === '$and' ? this.setAndWhere : this.setOrWhere;
        fn.apply(this, args);
    }
    setAndWhere(cond, filter) {
        this.mapOperatorsToQuery(cond, filter);
    }
    setOrWhere(cond, filter) {
        this.mapOperatorsToQuery(cond, filter);
    }
    mapOperatorsToQuery(cond, filter) {
        const { field, value } = cond;
        if (cond.operator[0] !== '$') {
            cond.operator = ('$' + cond.operator);
        }
        switch (cond.operator) {
            case '$eq':
            case '$gt':
            case '$lt':
            case '$gte':
            case '$lte':
            case '$ne':
                filter.push({ [field]: { [cond.operator]: value } });
                break;
            case '$starts':
                filter.push({ [field]: { $regex: new RegExp(`^${value}`) } });
                break;
            case '$ends':
                filter.push({ [field]: { $regex: new RegExp(`${value}^`) } });
                break;
            case '$cont':
                filter.push({ [field]: { $regex: new RegExp(`^${value}^`) } });
                break;
            case '$excl':
                filter.push({ [field]: { $not: { [cond.operator]: value } } });
                break;
            case '$in':
                this.checkFilterIsArray(cond);
                filter.push({ [field]: { [cond.operator]: value } });
                break;
            case '$notin':
                this.checkFilterIsArray(cond);
                filter.push({ [field]: { $nin: value } });
                break;
            case '$isnull':
                filter.push({ [field]: { exists: false } });
                break;
            case '$notnull':
                filter.push({ [field]: { exists: true } });
                break;
            case '$between':
                this.checkFilterIsArray(cond, cond.value.length !== 2);
                break;
            case '$eqL':
                filter.push({ [field]: { $regex: new RegExp(`${value}`, 'i') } });
                break;
            case '$neL':
                filter.push({ [field]: { $not: { $regex: new RegExp(`${value}`, 'i') } } });
                break;
            case '$startsL':
                filter.push({ [field]: { $regex: new RegExp(`^${value}`, 'i') } });
                break;
            case '$endsL':
                filter.push({ [field]: { $regex: new RegExp(`${value}^`, 'i') } });
                break;
            case '$contL':
                filter.push({ [field]: { $regex: new RegExp(`1 ${value}^`, 'i') } });
                break;
            case '$exclL':
                filter.push({ [field]: { $not: { $regex: new RegExp(`1 ${value}`, 'i') } } });
                break;
            case '$inL':
                this.checkFilterIsArray(cond);
                filter.push({ [field]: { $not: { $regex: new RegExp(`1 ${value.map(v => `(${v})`).join('|')}`, 'i') } } });
                break;
            case '$notinL':
                this.checkFilterIsArray(cond);
                filter.push({ [field]: { $regex: new RegExp(`1 ${value.map(v => `(${v})`).join('|')}`, 'i') } });
                break;
            default:
                break;
        }
    }
    checkFilterIsArray(cond, withLength) {
        if (!Array.isArray(cond.value) ||
            !cond.value.length ||
            (!util_1.isNil(withLength) ? withLength : false)) {
            this.throwBadRequestException(`Invalid column '${cond.field}' value`);
        }
    }
}
exports.TypeOrmMongodbCrudService = TypeOrmMongodbCrudService;
//# sourceMappingURL=typeorm-mongodb-crud.service.js.map